# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Import/Export zone files
"""

from typing import BinaryIO, Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class DnsRecordBulkV1(BaseService):
    """The DNS Record Bulk V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'dns_record_bulk'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_identifier: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'DnsRecordBulkV1':
        """
        Return a new client for the DNS Record Bulk service using the specified
               parameters and external configuration.

        :param str crn: Full url-encoded CRN of the service instance.

        :param str zone_identifier: Identifier of zone.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_identifier,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_identifier: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the DNS Record Bulk service.

        :param str crn: Full url-encoded CRN of the service instance.

        :param str zone_identifier: Identifier of zone.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_identifier = zone_identifier


    #########################
    # dNSRecords
    #########################


    def get_dns_records_bulk(self,
        **kwargs
    ) -> DetailedResponse:
        """
        Export zone file.

        Export zone file.

        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `BinaryIO` result
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_dns_records_bulk')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records_bulk'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def post_dns_records_bulk(self,
        *,
        file: BinaryIO = None,
        file_content_type: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Import zone file.

        Import zone file.

        :param BinaryIO file: (optional) file to upload.
        :param str file_content_type: (optional) The content type of file.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `DnsRecordsObject` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='post_dns_records_bulk')
        headers.update(sdk_headers)

        form_data = []
        if file:
            form_data.append(('file', (None, file, file_content_type or 'application/octet-stream')))

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/dns_records_bulk'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       files=form_data)

        response = self.send(request)
        return response


##############################################################################
# Models
##############################################################################


class DnsRecordsObjectMessagesItem():
    """
    DnsRecordsObjectMessagesItem.

    :attr int code: (optional) Message code.
    :attr str message: (optional) Message corresponding to the code.
    """

    def __init__(self,
                 *,
                 code: int = None,
                 message: str = None) -> None:
        """
        Initialize a DnsRecordsObjectMessagesItem object.

        :param int code: (optional) Message code.
        :param str message: (optional) Message corresponding to the code.
        """
        self.code = code
        self.message = message

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DnsRecordsObjectMessagesItem':
        """Initialize a DnsRecordsObjectMessagesItem object from a json dictionary."""
        args = {}
        if 'code' in _dict:
            args['code'] = _dict.get('code')
        if 'message' in _dict:
            args['message'] = _dict.get('message')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DnsRecordsObjectMessagesItem object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'code') and self.code is not None:
            _dict['code'] = self.code
        if hasattr(self, 'message') and self.message is not None:
            _dict['message'] = self.message
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DnsRecordsObjectMessagesItem object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DnsRecordsObjectMessagesItem') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DnsRecordsObjectMessagesItem') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class DnsRecordsObjectResult():
    """
    DNS record.

    :attr int recs_added: total records added.
    :attr int total_records_parsed: total records parsed.
    """

    def __init__(self,
                 recs_added: int,
                 total_records_parsed: int) -> None:
        """
        Initialize a DnsRecordsObjectResult object.

        :param int recs_added: total records added.
        :param int total_records_parsed: total records parsed.
        """
        self.recs_added = recs_added
        self.total_records_parsed = total_records_parsed

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DnsRecordsObjectResult':
        """Initialize a DnsRecordsObjectResult object from a json dictionary."""
        args = {}
        if 'recs_added' in _dict:
            args['recs_added'] = _dict.get('recs_added')
        else:
            raise ValueError('Required property \'recs_added\' not present in DnsRecordsObjectResult JSON')
        if 'total_records_parsed' in _dict:
            args['total_records_parsed'] = _dict.get('total_records_parsed')
        else:
            raise ValueError('Required property \'total_records_parsed\' not present in DnsRecordsObjectResult JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DnsRecordsObjectResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'recs_added') and self.recs_added is not None:
            _dict['recs_added'] = self.recs_added
        if hasattr(self, 'total_records_parsed') and self.total_records_parsed is not None:
            _dict['total_records_parsed'] = self.total_records_parsed
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DnsRecordsObjectResult object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DnsRecordsObjectResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DnsRecordsObjectResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class DnsRecordsObjectTiming():
    """
    timing object.

    :attr str start_time: (optional) start time.
    :attr str end_time: (optional) end time.
    :attr int process_time: (optional) process time.
    """

    def __init__(self,
                 *,
                 start_time: str = None,
                 end_time: str = None,
                 process_time: int = None) -> None:
        """
        Initialize a DnsRecordsObjectTiming object.

        :param str start_time: (optional) start time.
        :param str end_time: (optional) end time.
        :param int process_time: (optional) process time.
        """
        self.start_time = start_time
        self.end_time = end_time
        self.process_time = process_time

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DnsRecordsObjectTiming':
        """Initialize a DnsRecordsObjectTiming object from a json dictionary."""
        args = {}
        if 'start_time' in _dict:
            args['start_time'] = _dict.get('start_time')
        if 'end_time' in _dict:
            args['end_time'] = _dict.get('end_time')
        if 'process_time' in _dict:
            args['process_time'] = _dict.get('process_time')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DnsRecordsObjectTiming object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'start_time') and self.start_time is not None:
            _dict['start_time'] = self.start_time
        if hasattr(self, 'end_time') and self.end_time is not None:
            _dict['end_time'] = self.end_time
        if hasattr(self, 'process_time') and self.process_time is not None:
            _dict['process_time'] = self.process_time
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DnsRecordsObjectTiming object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DnsRecordsObjectTiming') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DnsRecordsObjectTiming') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class DnsRecordsObject():
    """
    dns records objects.

    :attr bool success: Operation success flag.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[DnsRecordsObjectMessagesItem] messages: Array of messages returned.
    :attr DnsRecordsObjectResult result: DNS record.
    :attr DnsRecordsObjectTiming timing: (optional) timing object.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List['DnsRecordsObjectMessagesItem'],
                 result: 'DnsRecordsObjectResult',
                 *,
                 timing: 'DnsRecordsObjectTiming' = None) -> None:
        """
        Initialize a DnsRecordsObject object.

        :param bool success: Operation success flag.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[DnsRecordsObjectMessagesItem] messages: Array of messages
               returned.
        :param DnsRecordsObjectResult result: DNS record.
        :param DnsRecordsObjectTiming timing: (optional) timing object.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result
        self.timing = timing

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DnsRecordsObject':
        """Initialize a DnsRecordsObject object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in DnsRecordsObject JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in DnsRecordsObject JSON')
        if 'messages' in _dict:
            args['messages'] = [DnsRecordsObjectMessagesItem.from_dict(x) for x in _dict.get('messages')]
        else:
            raise ValueError('Required property \'messages\' not present in DnsRecordsObject JSON')
        if 'result' in _dict:
            args['result'] = DnsRecordsObjectResult.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in DnsRecordsObject JSON')
        if 'timing' in _dict:
            args['timing'] = DnsRecordsObjectTiming.from_dict(_dict.get('timing'))
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DnsRecordsObject object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = [x.to_dict() for x in self.messages]
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        if hasattr(self, 'timing') and self.timing is not None:
            _dict['timing'] = self.timing.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DnsRecordsObject object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'DnsRecordsObject') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DnsRecordsObject') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Security Events API
"""

from datetime import datetime
from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class SecurityEventsApiV1(BaseService):
    """The Security Events API V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'security_events_api'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_id: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'SecurityEventsApiV1':
        """
        Return a new client for the Security Events API service using the specified
               parameters and external configuration.

        :param str crn: Full url-encoded cloud resource name (CRN) of resource
               instance.

        :param str zone_id: zone identifier.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_id is None:
            raise ValueError('zone_id must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_id,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_id: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the Security Events API service.

        :param str crn: Full url-encoded cloud resource name (CRN) of resource
               instance.

        :param str zone_id: zone identifier.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_id is None:
            raise ValueError('zone_id must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_id = zone_id


    #########################
    # Security Events
    #########################


    def security_events(self,
        *,
        ip_class: str = None,
        method: str = None,
        scheme: str = None,
        ip: str = None,
        host: str = None,
        proto: str = None,
        uri: str = None,
        ua: str = None,
        colo: str = None,
        ray_id: str = None,
        kind: str = None,
        action: str = None,
        cursor: str = None,
        country: str = None,
        since: datetime = None,
        source: str = None,
        limit: int = None,
        rule_id: str = None,
        until: datetime = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Logs of the mitigations performed by Firewall features.

        Provides a full log of the mitigations performed by the CIS Firewall features
        including; Firewall Rules, Rate Limiting, Security Level, Access Rules (IP, IP
        Range, ASN, and Country), WAF (Web Application Firewall), User Agent Blocking,
        Zone Lockdown, and Advanced DDoS Protection.

        :param str ip_class: (optional) IP class is a map of client IP to visitor
               classification.
        :param str method: (optional) The HTTP method of the request.
        :param str scheme: (optional) The scheme of the uri.
        :param str ip: (optional) The IPv4 or IPv6 address from which the request
               originated.
        :param str host: (optional) The hostname the request attempted to access.
        :param str proto: (optional) The protocol of the request.
        :param str uri: (optional) The URI requested from the hostname.
        :param str ua: (optional) The client user agent that initiated the request.
        :param str colo: (optional) The 3-letter CF PoP code.
        :param str ray_id: (optional) Ray ID of the request.
        :param str kind: (optional) Kind of events. Now it is only firewall.
        :param str action: (optional) What type of action was taken.
        :param str cursor: (optional) Cursor position and direction for requesting
               next set of records when amount of results was limited by the limit
               parameter. A valid value for the cursor can be obtained from the cursors
               object in the result_info structure.
        :param str country: (optional) The 2-digit country code in which the
               request originated.
        :param datetime since: (optional) Start date and time of requesting data
               period in the ISO8601 format. Can't go back more than a year.
        :param str source: (optional) Source of the event.
        :param int limit: (optional) The number of events to return.
        :param str rule_id: (optional) The ID of the rule that triggered the event,
               should be considered in the context of source.
        :param datetime until: (optional) End date and time of requesting data
               period in the ISO8601 format.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `SecurityEvents` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='security_events')
        headers.update(sdk_headers)

        params = {
            'ip_class': ip_class,
            'method': method,
            'scheme': scheme,
            'ip': ip,
            'host': host,
            'proto': proto,
            'uri': uri,
            'ua': ua,
            'colo': colo,
            'ray_id': ray_id,
            'kind': kind,
            'action': action,
            'cursor': cursor,
            'country': country,
            'since': since,
            'source': source,
            'limit': limit,
            'rule_id': rule_id,
            'until': until
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/security/events'.format(
            *self.encode_path_vars(self.crn, self.zone_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


class SecurityEventsEnums:
    """
    Enums for security_events parameters.
    """

    class IpClass(str, Enum):
        """
        IP class is a map of client IP to visitor classification.
        """
        UNKNOWN = 'unknown'
        CLEAN = 'clean'
        BADHOST = 'badHost'
        SEARCHENGINE = 'searchEngine'
        WHITELIST = 'whitelist'
        GREYLIST = 'greylist'
        MONITORINGSERVICE = 'monitoringService'
        SECURITYSCANNER = 'securityScanner'
        NORECORD = 'noRecord'
        SCAN = 'scan'
        BACKUPSERVICE = 'backupService'
        MOBILEPLATFORM = 'mobilePlatform'
        TOR = 'tor'
    class Method(str, Enum):
        """
        The HTTP method of the request.
        """
        GET = 'GET'
        POST = 'POST'
        DELETE = 'DELETE'
        PUT = 'PUT'
        HEAD = 'HEAD'
        PURGE = 'PURGE'
        OPTIONS = 'OPTIONS'
        PROPFIND = 'PROPFIND'
        MKCOL = 'MKCOL'
        PATCH = 'PATCH'
        ACL = 'ACL'
        BCOPY = 'BCOPY'
        BDELETE = 'BDELETE'
        BMOVE = 'BMOVE'
        BPROPFIND = 'BPROPFIND'
        BPROPPATCH = 'BPROPPATCH'
        CHECKIN = 'CHECKIN'
        CHECKOUT = 'CHECKOUT'
        CONNECT = 'CONNECT'
        COPY = 'COPY'
        LABEL = 'LABEL'
        LOCK = 'LOCK'
        MERGE = 'MERGE'
        MKACTIVITY = 'MKACTIVITY'
        MKWORKSPACE = 'MKWORKSPACE'
        MOVE = 'MOVE'
        NOTIFY = 'NOTIFY'
        ORDERPATCH = 'ORDERPATCH'
        POLL = 'POLL'
        PROPPATCH = 'PROPPATCH'
        REPORT = 'REPORT'
        SEARCH = 'SEARCH'
        SUBSCRIBE = 'SUBSCRIBE'
        TRACE = 'TRACE'
        UNCHECKOUT = 'UNCHECKOUT'
        UNLOCK = 'UNLOCK'
        UNSUBSCRIBE = 'UNSUBSCRIBE'
        UPDATE = 'UPDATE'
        VERSION_CONTROL = 'VERSION-CONTROL'
        BASELINE_CONTROL = 'BASELINE-CONTROL'
        X_MS_ENUMATTS = 'X-MS-ENUMATTS'
        RPC_OUT_DATA = 'RPC_OUT_DATA'
        RPC_IN_DATA = 'RPC_IN_DATA'
        JSON = 'JSON'
        COOK = 'COOK'
        TRACK = 'TRACK'
    class Scheme(str, Enum):
        """
        The scheme of the uri.
        """
        UNKNOWN = 'unknown'
        HTTP = 'http'
        HTTPS = 'https'
    class Proto(str, Enum):
        """
        The protocol of the request.
        """
        UNK = 'UNK'
        HTTP_1_0 = 'HTTP/1.0'
        HTTP_1_1 = 'HTTP/1.1'
        HTTP_1_2 = 'HTTP/1.2'
        HTTP_2 = 'HTTP/2'
        SPDY_3_1 = 'SPDY/3.1'
    class Kind(str, Enum):
        """
        Kind of events. Now it is only firewall.
        """
        FIREWALL = 'firewall'
    class Action(str, Enum):
        """
        What type of action was taken.
        """
        UNKNOWN = 'unknown'
        ALLOW = 'allow'
        DROP = 'drop'
        CHALLENGE = 'challenge'
        JSCHALLENGE = 'jschallenge'
        SIMULATE = 'simulate'
        CONNECTIONCLOSE = 'connectionClose'
        LOG = 'log'
    class Source(str, Enum):
        """
        Source of the event.
        """
        UNKNOWN = 'unknown'
        ASN = 'asn'
        COUNTRY = 'country'
        IP = 'ip'
        IPRANGE = 'ipRange'
        SECURITYLEVEL = 'securityLevel'
        ZONELOCKDOWN = 'zoneLockdown'
        WAF = 'waf'
        UABLOCK = 'uaBlock'
        RATELIMIT = 'rateLimit'
        FIREWALLRULES = 'firewallRules'
        BIC = 'bic'
        HOT = 'hot'
        L7DDOS = 'l7ddos'


##############################################################################
# Models
##############################################################################


class ResultInfoCursors():
    """
    Cursor positions of the security events.

    :attr str after: The events in the response is after this cursor position.
    :attr str before: The events in the response is before this cursor position.
    """

    def __init__(self,
                 after: str,
                 before: str) -> None:
        """
        Initialize a ResultInfoCursors object.

        :param str after: The events in the response is after this cursor position.
        :param str before: The events in the response is before this cursor
               position.
        """
        self.after = after
        self.before = before

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResultInfoCursors':
        """Initialize a ResultInfoCursors object from a json dictionary."""
        args = {}
        if 'after' in _dict:
            args['after'] = _dict.get('after')
        else:
            raise ValueError('Required property \'after\' not present in ResultInfoCursors JSON')
        if 'before' in _dict:
            args['before'] = _dict.get('before')
        else:
            raise ValueError('Required property \'before\' not present in ResultInfoCursors JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResultInfoCursors object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'after') and self.after is not None:
            _dict['after'] = self.after
        if hasattr(self, 'before') and self.before is not None:
            _dict['before'] = self.before
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResultInfoCursors object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResultInfoCursors') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResultInfoCursors') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ResultInfoScannedRange():
    """
    The time window of the events.

    :attr str since: Start date and time of the events.
    :attr str until: End date and time of the events.
    """

    def __init__(self,
                 since: str,
                 until: str) -> None:
        """
        Initialize a ResultInfoScannedRange object.

        :param str since: Start date and time of the events.
        :param str until: End date and time of the events.
        """
        self.since = since
        self.until = until

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResultInfoScannedRange':
        """Initialize a ResultInfoScannedRange object from a json dictionary."""
        args = {}
        if 'since' in _dict:
            args['since'] = _dict.get('since')
        else:
            raise ValueError('Required property \'since\' not present in ResultInfoScannedRange JSON')
        if 'until' in _dict:
            args['until'] = _dict.get('until')
        else:
            raise ValueError('Required property \'until\' not present in ResultInfoScannedRange JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResultInfoScannedRange object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'since') and self.since is not None:
            _dict['since'] = self.since
        if hasattr(self, 'until') and self.until is not None:
            _dict['until'] = self.until
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResultInfoScannedRange object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResultInfoScannedRange') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResultInfoScannedRange') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class SecurityEventObjectMatchesItem():
    """
    SecurityEventObjectMatchesItem.

    :attr str rule_id: The ID of the rule that triggered the event, should be
          considered in the context of source.
    :attr str source: Source of the event.
    :attr str action: What type of action was taken.
    :attr object metadata: metadata.
    """

    def __init__(self,
                 rule_id: str,
                 source: str,
                 action: str,
                 metadata: object) -> None:
        """
        Initialize a SecurityEventObjectMatchesItem object.

        :param str rule_id: The ID of the rule that triggered the event, should be
               considered in the context of source.
        :param str source: Source of the event.
        :param str action: What type of action was taken.
        :param object metadata: metadata.
        """
        self.rule_id = rule_id
        self.source = source
        self.action = action
        self.metadata = metadata

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'SecurityEventObjectMatchesItem':
        """Initialize a SecurityEventObjectMatchesItem object from a json dictionary."""
        args = {}
        if 'rule_id' in _dict:
            args['rule_id'] = _dict.get('rule_id')
        else:
            raise ValueError('Required property \'rule_id\' not present in SecurityEventObjectMatchesItem JSON')
        if 'source' in _dict:
            args['source'] = _dict.get('source')
        else:
            raise ValueError('Required property \'source\' not present in SecurityEventObjectMatchesItem JSON')
        if 'action' in _dict:
            args['action'] = _dict.get('action')
        else:
            raise ValueError('Required property \'action\' not present in SecurityEventObjectMatchesItem JSON')
        if 'metadata' in _dict:
            args['metadata'] = _dict.get('metadata')
        else:
            raise ValueError('Required property \'metadata\' not present in SecurityEventObjectMatchesItem JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a SecurityEventObjectMatchesItem object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'rule_id') and self.rule_id is not None:
            _dict['rule_id'] = self.rule_id
        if hasattr(self, 'source') and self.source is not None:
            _dict['source'] = self.source
        if hasattr(self, 'action') and self.action is not None:
            _dict['action'] = self.action
        if hasattr(self, 'metadata') and self.metadata is not None:
            _dict['metadata'] = self.metadata
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this SecurityEventObjectMatchesItem object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'SecurityEventObjectMatchesItem') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'SecurityEventObjectMatchesItem') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ResultInfo():
    """
    Statistics of results.

    :attr ResultInfoCursors cursors: Cursor positions of the security events.
    :attr ResultInfoScannedRange scanned_range: The time window of the events.
    """

    def __init__(self,
                 cursors: 'ResultInfoCursors',
                 scanned_range: 'ResultInfoScannedRange') -> None:
        """
        Initialize a ResultInfo object.

        :param ResultInfoCursors cursors: Cursor positions of the security events.
        :param ResultInfoScannedRange scanned_range: The time window of the events.
        """
        self.cursors = cursors
        self.scanned_range = scanned_range

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ResultInfo':
        """Initialize a ResultInfo object from a json dictionary."""
        args = {}
        if 'cursors' in _dict:
            args['cursors'] = ResultInfoCursors.from_dict(_dict.get('cursors'))
        else:
            raise ValueError('Required property \'cursors\' not present in ResultInfo JSON')
        if 'scanned_range' in _dict:
            args['scanned_range'] = ResultInfoScannedRange.from_dict(_dict.get('scanned_range'))
        else:
            raise ValueError('Required property \'scanned_range\' not present in ResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'cursors') and self.cursors is not None:
            _dict['cursors'] = self.cursors.to_dict()
        if hasattr(self, 'scanned_range') and self.scanned_range is not None:
            _dict['scanned_range'] = self.scanned_range.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class SecurityEventObject():
    """
    Security event object.

    :attr str ray_id: Ray ID of the request.
    :attr str kind: Kind of events. Now it is only firewall.
    :attr str source: Source of the event.
    :attr str action: What type of action was taken.
    :attr str rule_id: The ID of the rule that triggered the event, should be
          considered in the context of source.
    :attr str ip: The IPv4 or IPv6 address from which the request originated.
    :attr str ip_class: IP class is a map of client IP to visitor classification.
    :attr str country: The 2-digit country code in which the request originated.
    :attr str colo: The 3-letter CF PoP code.
    :attr str host: The hostname the request attempted to access.
    :attr str method: The HTTP method of the request.
    :attr str proto: The protocol of the request.
    :attr str scheme: The scheme of the uri.
    :attr str ua: The client user agent that initiated the request.
    :attr str uri: The URI requested from the hostname.
    :attr datetime occurred_at: The time that the event occurred.
    :attr List[SecurityEventObjectMatchesItem] matches: The firewall rules those the
          event matches.
    """

    def __init__(self,
                 ray_id: str,
                 kind: str,
                 source: str,
                 action: str,
                 rule_id: str,
                 ip: str,
                 ip_class: str,
                 country: str,
                 colo: str,
                 host: str,
                 method: str,
                 proto: str,
                 scheme: str,
                 ua: str,
                 uri: str,
                 occurred_at: datetime,
                 matches: List['SecurityEventObjectMatchesItem']) -> None:
        """
        Initialize a SecurityEventObject object.

        :param str ray_id: Ray ID of the request.
        :param str kind: Kind of events. Now it is only firewall.
        :param str source: Source of the event.
        :param str action: What type of action was taken.
        :param str rule_id: The ID of the rule that triggered the event, should be
               considered in the context of source.
        :param str ip: The IPv4 or IPv6 address from which the request originated.
        :param str ip_class: IP class is a map of client IP to visitor
               classification.
        :param str country: The 2-digit country code in which the request
               originated.
        :param str colo: The 3-letter CF PoP code.
        :param str host: The hostname the request attempted to access.
        :param str method: The HTTP method of the request.
        :param str proto: The protocol of the request.
        :param str scheme: The scheme of the uri.
        :param str ua: The client user agent that initiated the request.
        :param str uri: The URI requested from the hostname.
        :param datetime occurred_at: The time that the event occurred.
        :param List[SecurityEventObjectMatchesItem] matches: The firewall rules
               those the event matches.
        """
        self.ray_id = ray_id
        self.kind = kind
        self.source = source
        self.action = action
        self.rule_id = rule_id
        self.ip = ip
        self.ip_class = ip_class
        self.country = country
        self.colo = colo
        self.host = host
        self.method = method
        self.proto = proto
        self.scheme = scheme
        self.ua = ua
        self.uri = uri
        self.occurred_at = occurred_at
        self.matches = matches

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'SecurityEventObject':
        """Initialize a SecurityEventObject object from a json dictionary."""
        args = {}
        if 'ray_id' in _dict:
            args['ray_id'] = _dict.get('ray_id')
        else:
            raise ValueError('Required property \'ray_id\' not present in SecurityEventObject JSON')
        if 'kind' in _dict:
            args['kind'] = _dict.get('kind')
        else:
            raise ValueError('Required property \'kind\' not present in SecurityEventObject JSON')
        if 'source' in _dict:
            args['source'] = _dict.get('source')
        else:
            raise ValueError('Required property \'source\' not present in SecurityEventObject JSON')
        if 'action' in _dict:
            args['action'] = _dict.get('action')
        else:
            raise ValueError('Required property \'action\' not present in SecurityEventObject JSON')
        if 'rule_id' in _dict:
            args['rule_id'] = _dict.get('rule_id')
        else:
            raise ValueError('Required property \'rule_id\' not present in SecurityEventObject JSON')
        if 'ip' in _dict:
            args['ip'] = _dict.get('ip')
        else:
            raise ValueError('Required property \'ip\' not present in SecurityEventObject JSON')
        if 'ip_class' in _dict:
            args['ip_class'] = _dict.get('ip_class')
        else:
            raise ValueError('Required property \'ip_class\' not present in SecurityEventObject JSON')
        if 'country' in _dict:
            args['country'] = _dict.get('country')
        else:
            raise ValueError('Required property \'country\' not present in SecurityEventObject JSON')
        if 'colo' in _dict:
            args['colo'] = _dict.get('colo')
        else:
            raise ValueError('Required property \'colo\' not present in SecurityEventObject JSON')
        if 'host' in _dict:
            args['host'] = _dict.get('host')
        else:
            raise ValueError('Required property \'host\' not present in SecurityEventObject JSON')
        if 'method' in _dict:
            args['method'] = _dict.get('method')
        else:
            raise ValueError('Required property \'method\' not present in SecurityEventObject JSON')
        if 'proto' in _dict:
            args['proto'] = _dict.get('proto')
        else:
            raise ValueError('Required property \'proto\' not present in SecurityEventObject JSON')
        if 'scheme' in _dict:
            args['scheme'] = _dict.get('scheme')
        else:
            raise ValueError('Required property \'scheme\' not present in SecurityEventObject JSON')
        if 'ua' in _dict:
            args['ua'] = _dict.get('ua')
        else:
            raise ValueError('Required property \'ua\' not present in SecurityEventObject JSON')
        if 'uri' in _dict:
            args['uri'] = _dict.get('uri')
        else:
            raise ValueError('Required property \'uri\' not present in SecurityEventObject JSON')
        if 'occurred_at' in _dict:
            args['occurred_at'] = string_to_datetime(_dict.get('occurred_at'))
        else:
            raise ValueError('Required property \'occurred_at\' not present in SecurityEventObject JSON')
        if 'matches' in _dict:
            args['matches'] = [SecurityEventObjectMatchesItem.from_dict(x) for x in _dict.get('matches')]
        else:
            raise ValueError('Required property \'matches\' not present in SecurityEventObject JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a SecurityEventObject object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'ray_id') and self.ray_id is not None:
            _dict['ray_id'] = self.ray_id
        if hasattr(self, 'kind') and self.kind is not None:
            _dict['kind'] = self.kind
        if hasattr(self, 'source') and self.source is not None:
            _dict['source'] = self.source
        if hasattr(self, 'action') and self.action is not None:
            _dict['action'] = self.action
        if hasattr(self, 'rule_id') and self.rule_id is not None:
            _dict['rule_id'] = self.rule_id
        if hasattr(self, 'ip') and self.ip is not None:
            _dict['ip'] = self.ip
        if hasattr(self, 'ip_class') and self.ip_class is not None:
            _dict['ip_class'] = self.ip_class
        if hasattr(self, 'country') and self.country is not None:
            _dict['country'] = self.country
        if hasattr(self, 'colo') and self.colo is not None:
            _dict['colo'] = self.colo
        if hasattr(self, 'host') and self.host is not None:
            _dict['host'] = self.host
        if hasattr(self, 'method') and self.method is not None:
            _dict['method'] = self.method
        if hasattr(self, 'proto') and self.proto is not None:
            _dict['proto'] = self.proto
        if hasattr(self, 'scheme') and self.scheme is not None:
            _dict['scheme'] = self.scheme
        if hasattr(self, 'ua') and self.ua is not None:
            _dict['ua'] = self.ua
        if hasattr(self, 'uri') and self.uri is not None:
            _dict['uri'] = self.uri
        if hasattr(self, 'occurred_at') and self.occurred_at is not None:
            _dict['occurred_at'] = datetime_to_string(self.occurred_at)
        if hasattr(self, 'matches') and self.matches is not None:
            _dict['matches'] = [x.to_dict() for x in self.matches]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this SecurityEventObject object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'SecurityEventObject') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'SecurityEventObject') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class KindEnum(str, Enum):
        """
        Kind of events. Now it is only firewall.
        """
        FIREWALL = 'firewall'


    class SourceEnum(str, Enum):
        """
        Source of the event.
        """
        UNKNOWN = 'unknown'
        ASN = 'asn'
        COUNTRY = 'country'
        IP = 'ip'
        IPRANGE = 'ipRange'
        SECURITYLEVEL = 'securityLevel'
        ZONELOCKDOWN = 'zoneLockdown'
        WAF = 'waf'
        UABLOCK = 'uaBlock'
        RATELIMIT = 'rateLimit'
        FIREWALLRULES = 'firewallRules'
        BIC = 'bic'
        HOT = 'hot'
        L7DDOS = 'l7ddos'


    class ActionEnum(str, Enum):
        """
        What type of action was taken.
        """
        UNKNOWN = 'unknown'
        ALLOW = 'allow'
        DROP = 'drop'
        CHALLENGE = 'challenge'
        JSCHALLENGE = 'jschallenge'
        SIMULATE = 'simulate'
        CONNECTIONCLOSE = 'connectionClose'
        LOG = 'log'


    class IpClassEnum(str, Enum):
        """
        IP class is a map of client IP to visitor classification.
        """
        UNKNOWN = 'unknown'
        CLEAN = 'clean'
        BADHOST = 'badHost'
        SEARCHENGINE = 'searchEngine'
        WHITELIST = 'whitelist'
        GREYLIST = 'greylist'
        MONITORINGSERVICE = 'monitoringService'
        SECURITYSCANNER = 'securityScanner'
        NORECORD = 'noRecord'
        SCAN = 'scan'
        BACKUPSERVICE = 'backupService'
        MOBILEPLATFORM = 'mobilePlatform'
        TOR = 'tor'


    class MethodEnum(str, Enum):
        """
        The HTTP method of the request.
        """
        GET = 'GET'
        POST = 'POST'
        DELETE = 'DELETE'
        PUT = 'PUT'
        HEAD = 'HEAD'
        PURGE = 'PURGE'
        OPTIONS = 'OPTIONS'
        PROPFIND = 'PROPFIND'
        MKCOL = 'MKCOL'
        PATCH = 'PATCH'
        ACL = 'ACL'
        BCOPY = 'BCOPY'
        BDELETE = 'BDELETE'
        BMOVE = 'BMOVE'
        BPROPFIND = 'BPROPFIND'
        BPROPPATCH = 'BPROPPATCH'
        CHECKIN = 'CHECKIN'
        CHECKOUT = 'CHECKOUT'
        CONNECT = 'CONNECT'
        COPY = 'COPY'
        LABEL = 'LABEL'
        LOCK = 'LOCK'
        MERGE = 'MERGE'
        MKACTIVITY = 'MKACTIVITY'
        MKWORKSPACE = 'MKWORKSPACE'
        MOVE = 'MOVE'
        NOTIFY = 'NOTIFY'
        ORDERPATCH = 'ORDERPATCH'
        POLL = 'POLL'
        PROPPATCH = 'PROPPATCH'
        REPORT = 'REPORT'
        SEARCH = 'SEARCH'
        SUBSCRIBE = 'SUBSCRIBE'
        TRACE = 'TRACE'
        UNCHECKOUT = 'UNCHECKOUT'
        UNLOCK = 'UNLOCK'
        UNSUBSCRIBE = 'UNSUBSCRIBE'
        UPDATE = 'UPDATE'
        VERSION_CONTROL = 'VERSION-CONTROL'
        BASELINE_CONTROL = 'BASELINE-CONTROL'
        X_MS_ENUMATTS = 'X-MS-ENUMATTS'
        RPC_OUT_DATA = 'RPC_OUT_DATA'
        RPC_IN_DATA = 'RPC_IN_DATA'
        JSON = 'JSON'
        COOK = 'COOK'
        TRACK = 'TRACK'


    class ProtoEnum(str, Enum):
        """
        The protocol of the request.
        """
        UNK = 'UNK'
        HTTP_1_0 = 'HTTP/1.0'
        HTTP_1_1 = 'HTTP/1.1'
        HTTP_1_2 = 'HTTP/1.2'
        HTTP_2 = 'HTTP/2'
        SPDY_3_1 = 'SPDY/3.1'


    class SchemeEnum(str, Enum):
        """
        The scheme of the uri.
        """
        UNKNOWN = 'unknown'
        HTTP = 'http'
        HTTPS = 'https'


class SecurityEvents():
    """
    security events objects.

    :attr List[SecurityEventObject] result: Container for response information.
    :attr ResultInfo result_info: Statistics of results.
    :attr bool success: Was the get successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    """

    def __init__(self,
                 result: List['SecurityEventObject'],
                 result_info: 'ResultInfo',
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]]) -> None:
        """
        Initialize a SecurityEvents object.

        :param List[SecurityEventObject] result: Container for response
               information.
        :param ResultInfo result_info: Statistics of results.
        :param bool success: Was the get successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        """
        self.result = result
        self.result_info = result_info
        self.success = success
        self.errors = errors
        self.messages = messages

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'SecurityEvents':
        """Initialize a SecurityEvents object from a json dictionary."""
        args = {}
        if 'result' in _dict:
            args['result'] = [SecurityEventObject.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in SecurityEvents JSON')
        if 'result_info' in _dict:
            args['result_info'] = ResultInfo.from_dict(_dict.get('result_info'))
        else:
            raise ValueError('Required property \'result_info\' not present in SecurityEvents JSON')
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in SecurityEvents JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in SecurityEvents JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in SecurityEvents JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a SecurityEvents object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        if hasattr(self, 'result_info') and self.result_info is not None:
            _dict['result_info'] = self.result_info.to_dict()
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this SecurityEvents object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'SecurityEvents') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'SecurityEvents') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This document describes CIS Pagerule API.
"""

from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import convert_model

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class PageRuleApiV1(BaseService):
    """The Page Rule API V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'page_rule_api'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_id: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'PageRuleApiV1':
        """
        Return a new client for the Page Rule API service using the specified
               parameters and external configuration.

        :param str crn: instance id.

        :param str zone_id: zone id.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_id is None:
            raise ValueError('zone_id must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_id,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_id: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the Page Rule API service.

        :param str crn: instance id.

        :param str zone_id: zone id.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_id is None:
            raise ValueError('zone_id must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_id = zone_id


    #########################
    # Page Rules
    #########################


    def get_page_rule(self,
        rule_id: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get page rule.

        Get a page rule details.

        :param str rule_id: rule id.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PageRulesResponseWithoutResultInfo` object
        """

        if rule_id is None:
            raise ValueError('rule_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_page_rule')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/pagerules/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_id, rule_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def change_page_rule(self,
        rule_id: str,
        *,
        targets: List['TargetsItem'] = None,
        actions: List['PageRulesBodyActionsItem'] = None,
        priority: int = None,
        status: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Change page rule.

        Change a page rule.

        :param str rule_id: rule id.
        :param List[TargetsItem] targets: (optional) targets.
        :param List[PageRulesBodyActionsItem] actions: (optional) actions.
        :param int priority: (optional) priority.
        :param str status: (optional) status.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PageRulesResponseWithoutResultInfo` object
        """

        if rule_id is None:
            raise ValueError('rule_id must be provided')
        if targets is not None:
            targets = [convert_model(x) for x in targets]
        if actions is not None:
            actions = [convert_model(x) for x in actions]
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='change_page_rule')
        headers.update(sdk_headers)

        data = {
            'targets': targets,
            'actions': actions,
            'priority': priority,
            'status': status
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/pagerules/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_id, rule_id))
        request = self.prepare_request(method='PATCH',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def update_page_rule(self,
        rule_id: str,
        *,
        targets: List['TargetsItem'] = None,
        actions: List['PageRulesBodyActionsItem'] = None,
        priority: int = None,
        status: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update page rule.

        Replace a page rule. The final rule will exactly match the data passed with this
        request.

        :param str rule_id: rule id.
        :param List[TargetsItem] targets: (optional) targets.
        :param List[PageRulesBodyActionsItem] actions: (optional) actions.
        :param int priority: (optional) priority.
        :param str status: (optional) status.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PageRulesResponseWithoutResultInfo` object
        """

        if rule_id is None:
            raise ValueError('rule_id must be provided')
        if targets is not None:
            targets = [convert_model(x) for x in targets]
        if actions is not None:
            actions = [convert_model(x) for x in actions]
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_page_rule')
        headers.update(sdk_headers)

        data = {
            'targets': targets,
            'actions': actions,
            'priority': priority,
            'status': status
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/pagerules/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_id, rule_id))
        request = self.prepare_request(method='PUT',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def delete_page_rule(self,
        rule_id: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Delete page rule.

        Delete a page rule.

        :param str rule_id: rule id.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PageRulesDeleteResponse` object
        """

        if rule_id is None:
            raise ValueError('rule_id must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='delete_page_rule')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/pagerules/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_id, rule_id))
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def list_page_rules(self,
        *,
        status: str = None,
        order: str = None,
        direction: str = None,
        match: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        List page rules.

        List page rules.

        :param str status: (optional) default value: disabled. valid values:
               active, disabled.
        :param str order: (optional) default value: priority. valid values: status,
               priority.
        :param str direction: (optional) default value: desc. valid values: asc,
               desc.
        :param str match: (optional) default value: all. valid values: any, all.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PageRulesResponseListAll` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_page_rules')
        headers.update(sdk_headers)

        params = {
            'status': status,
            'order': order,
            'direction': direction,
            'match': match
        }

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/pagerules'.format(
            *self.encode_path_vars(self.crn, self.zone_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


    def create_page_rule(self,
        *,
        targets: List['TargetsItem'] = None,
        actions: List['PageRulesBodyActionsItem'] = None,
        priority: int = None,
        status: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Create page rule.

        Create a page rule.

        :param List[TargetsItem] targets: (optional) targets.
        :param List[PageRulesBodyActionsItem] actions: (optional) actions.
        :param int priority: (optional) priority.
        :param str status: (optional) status.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `PageRulesResponseWithoutResultInfo` object
        """

        if targets is not None:
            targets = [convert_model(x) for x in targets]
        if actions is not None:
            actions = [convert_model(x) for x in actions]
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='create_page_rule')
        headers.update(sdk_headers)

        data = {
            'targets': targets,
            'actions': actions,
            'priority': priority,
            'status': status
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/pagerules'.format(
            *self.encode_path_vars(self.crn, self.zone_id))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


##############################################################################
# Models
##############################################################################


class PageRulesBodyActionsItem():
    """
    PageRulesBodyActionsItem.

    :attr str id: " Page rule action field map from UI to API
              CF-UI                    map             API,
          'Disable Security'           to        'disable_security',
          'Browser Integrity Check'    to        'browser_check',
          'Server Side Excludes'       to        'server_side_exclude',
          'SSL'                        to        'ssl',
          'Browser Cache TTL'          to        'browser_cache_ttl',
          'Security Level'             to        'security_level',
          'Cache Level'                to        'cache_level',
          'Edge Cache TTL'             to        'edge_cache_ttl'
          'IP Geolocation Header'      to        'ip_geolocation,
          'Email Obfuscation'          to        'email_obfuscation',
          'Automatic HTTPS Rewrites'   to        'automatic_https_rewrites',
          'Opportunistic Encryption'   to        'opportunistic_encryption',
          'Forwarding URL'             to        'forwarding_url',
          'Always Use HTTPS'           to        'always_use_https',
          'Origin Cache Control'       to        'explicit_cache_control',
          'Bypass Cache on Cookie'     to        'bypass_cache_on_cookie',
          'Cache Deception Armor'      to        'cache_deception_armor',
          'WAF'                        to        'waf'
                            Page rule conflict list
          "forwarding_url"             with     all other settings for the rules
          "always_use_https"           with     all other settings for the rules
          "disable_security"           with     "email_obfuscation",
          "server_side_exclude", "waf"
          ".
    :attr object value: (optional) value.
    """

    def __init__(self,
                 id: str,
                 *,
                 value: object = None) -> None:
        """
        Initialize a PageRulesBodyActionsItem object.

        :param str id: " Page rule action field map from UI to API
                   CF-UI                    map             API,
               'Disable Security'           to        'disable_security',
               'Browser Integrity Check'    to        'browser_check',
               'Server Side Excludes'       to        'server_side_exclude',
               'SSL'                        to        'ssl',
               'Browser Cache TTL'          to        'browser_cache_ttl',
               'Security Level'             to        'security_level',
               'Cache Level'                to        'cache_level',
               'Edge Cache TTL'             to        'edge_cache_ttl'
               'IP Geolocation Header'      to        'ip_geolocation,
               'Email Obfuscation'          to        'email_obfuscation',
               'Automatic HTTPS Rewrites'   to        'automatic_https_rewrites',
               'Opportunistic Encryption'   to        'opportunistic_encryption',
               'Forwarding URL'             to        'forwarding_url',
               'Always Use HTTPS'           to        'always_use_https',
               'Origin Cache Control'       to        'explicit_cache_control',
               'Bypass Cache on Cookie'     to        'bypass_cache_on_cookie',
               'Cache Deception Armor'      to        'cache_deception_armor',
               'WAF'                        to        'waf'
                                 Page rule conflict list
               "forwarding_url"             with     all other settings for the rules
               "always_use_https"           with     all other settings for the rules
               "disable_security"           with     "email_obfuscation",
               "server_side_exclude", "waf"
               ".
        :param object value: (optional) value.
        """
        self.id = id
        self.value = value

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PageRulesBodyActionsItem':
        """Initialize a PageRulesBodyActionsItem object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in PageRulesBodyActionsItem JSON')
        if 'value' in _dict:
            args['value'] = _dict.get('value')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PageRulesBodyActionsItem object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'value') and self.value is not None:
            _dict['value'] = self.value
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PageRulesBodyActionsItem object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PageRulesBodyActionsItem') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PageRulesBodyActionsItem') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class IdEnum(str, Enum):
        """
        " Page rule action field map from UI to API
            CF-UI                    map             API,
        'Disable Security'           to        'disable_security',
        'Browser Integrity Check'    to        'browser_check',
        'Server Side Excludes'       to        'server_side_exclude',
        'SSL'                        to        'ssl',
        'Browser Cache TTL'          to        'browser_cache_ttl',
        'Security Level'             to        'security_level',
        'Cache Level'                to        'cache_level',
        'Edge Cache TTL'             to        'edge_cache_ttl'
        'IP Geolocation Header'      to        'ip_geolocation,
        'Email Obfuscation'          to        'email_obfuscation',
        'Automatic HTTPS Rewrites'   to        'automatic_https_rewrites',
        'Opportunistic Encryption'   to        'opportunistic_encryption',
        'Forwarding URL'             to        'forwarding_url',
        'Always Use HTTPS'           to        'always_use_https',
        'Origin Cache Control'       to        'explicit_cache_control',
        'Bypass Cache on Cookie'     to        'bypass_cache_on_cookie',
        'Cache Deception Armor'      to        'cache_deception_armor',
        'WAF'                        to        'waf'
                          Page rule conflict list
        "forwarding_url"             with     all other settings for the rules
        "always_use_https"           with     all other settings for the rules
        "disable_security"           with     "email_obfuscation", "server_side_exclude",
        "waf"
        ".
        """
        DISABLE_SECURITY = 'disable_security'
        BROWSER_CHECK = 'browser_check'
        SERVER_SIDE_EXCLUDE = 'server_side_exclude'
        SSL = 'ssl'
        BROWSER_CACHE_TTL = 'browser_cache_ttl'
        SECURITY_LEVEL = 'security_level'
        CACHE_LEVEL = 'cache_level'
        EDGE_CACHE_TTL = 'edge_cache_ttl'
        IP_GEOLOCATION = 'ip_geolocation'
        EMAIL_OBFUSCATION = 'email_obfuscation'
        AUTOMATIC_HTTPS_REWRITES = 'automatic_https_rewrites'
        OPPORTUNISTIC_ENCRYPTION = 'opportunistic_encryption'
        FORWARDING_URL = 'forwarding_url'
        ALWAYS_USE_HTTPS = 'always_use_https'
        EXPLICIT_CACHE_CONTROL = 'explicit_cache_control'
        BYPASS_CACHE_ON_COOKIE = 'bypass_cache_on_cookie'
        CACHE_DECEPTION_ARMOR = 'cache_deception_armor'
        WAF = 'waf'


class PageRulesDeleteResponseResult():
    """
    result.

    :attr str id: identifier.
    """

    def __init__(self,
                 id: str) -> None:
        """
        Initialize a PageRulesDeleteResponseResult object.

        :param str id: identifier.
        """
        self.id = id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PageRulesDeleteResponseResult':
        """Initialize a PageRulesDeleteResponseResult object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in PageRulesDeleteResponseResult JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PageRulesDeleteResponseResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PageRulesDeleteResponseResult object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PageRulesDeleteResponseResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PageRulesDeleteResponseResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class TargetsItem():
    """
    items.

    :attr str target: target.
    :attr TargetsItemConstraint constraint: constraint.
    """

    def __init__(self,
                 target: str,
                 constraint: 'TargetsItemConstraint') -> None:
        """
        Initialize a TargetsItem object.

        :param str target: target.
        :param TargetsItemConstraint constraint: constraint.
        """
        self.target = target
        self.constraint = constraint

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'TargetsItem':
        """Initialize a TargetsItem object from a json dictionary."""
        args = {}
        if 'target' in _dict:
            args['target'] = _dict.get('target')
        else:
            raise ValueError('Required property \'target\' not present in TargetsItem JSON')
        if 'constraint' in _dict:
            args['constraint'] = TargetsItemConstraint.from_dict(_dict.get('constraint'))
        else:
            raise ValueError('Required property \'constraint\' not present in TargetsItem JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a TargetsItem object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'target') and self.target is not None:
            _dict['target'] = self.target
        if hasattr(self, 'constraint') and self.constraint is not None:
            _dict['constraint'] = self.constraint.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this TargetsItem object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'TargetsItem') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'TargetsItem') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class TargetsItemConstraint():
    """
    constraint.

    :attr str operator: operator.
    :attr str value: value.
    """

    def __init__(self,
                 operator: str,
                 value: str) -> None:
        """
        Initialize a TargetsItemConstraint object.

        :param str operator: operator.
        :param str value: value.
        """
        self.operator = operator
        self.value = value

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'TargetsItemConstraint':
        """Initialize a TargetsItemConstraint object from a json dictionary."""
        args = {}
        if 'operator' in _dict:
            args['operator'] = _dict.get('operator')
        else:
            raise ValueError('Required property \'operator\' not present in TargetsItemConstraint JSON')
        if 'value' in _dict:
            args['value'] = _dict.get('value')
        else:
            raise ValueError('Required property \'value\' not present in TargetsItemConstraint JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a TargetsItemConstraint object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'operator') and self.operator is not None:
            _dict['operator'] = self.operator
        if hasattr(self, 'value') and self.value is not None:
            _dict['value'] = self.value
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this TargetsItemConstraint object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'TargetsItemConstraint') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'TargetsItemConstraint') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class PageRulesDeleteResponse():
    """
    page rules delete response.

    :attr bool success: success.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr PageRulesDeleteResponseResult result: result.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'PageRulesDeleteResponseResult') -> None:
        """
        Initialize a PageRulesDeleteResponse object.

        :param bool success: success.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param PageRulesDeleteResponseResult result: result.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PageRulesDeleteResponse':
        """Initialize a PageRulesDeleteResponse object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in PageRulesDeleteResponse JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in PageRulesDeleteResponse JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in PageRulesDeleteResponse JSON')
        if 'result' in _dict:
            args['result'] = PageRulesDeleteResponseResult.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in PageRulesDeleteResponse JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PageRulesDeleteResponse object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PageRulesDeleteResponse object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PageRulesDeleteResponse') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PageRulesDeleteResponse') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class PageRulesResponseListAll():
    """
    page rule response list all.

    :attr bool success: success.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr List[PageRuleResult] result: result.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: List['PageRuleResult']) -> None:
        """
        Initialize a PageRulesResponseListAll object.

        :param bool success: success.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param List[PageRuleResult] result: result.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PageRulesResponseListAll':
        """Initialize a PageRulesResponseListAll object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in PageRulesResponseListAll JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in PageRulesResponseListAll JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in PageRulesResponseListAll JSON')
        if 'result' in _dict:
            args['result'] = [PageRuleResult.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in PageRulesResponseListAll JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PageRulesResponseListAll object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PageRulesResponseListAll object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PageRulesResponseListAll') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PageRulesResponseListAll') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class PageRulesResponseWithoutResultInfo():
    """
    page rule response without result information.

    :attr bool success: success.
    :attr List[List[str]] errors: errors.
    :attr List[List[str]] messages: messages.
    :attr PageRuleResult result: page rule result.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'PageRuleResult') -> None:
        """
        Initialize a PageRulesResponseWithoutResultInfo object.

        :param bool success: success.
        :param List[List[str]] errors: errors.
        :param List[List[str]] messages: messages.
        :param PageRuleResult result: page rule result.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PageRulesResponseWithoutResultInfo':
        """Initialize a PageRulesResponseWithoutResultInfo object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in PageRulesResponseWithoutResultInfo JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in PageRulesResponseWithoutResultInfo JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in PageRulesResponseWithoutResultInfo JSON')
        if 'result' in _dict:
            args['result'] = PageRuleResult.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in PageRulesResponseWithoutResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PageRulesResponseWithoutResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PageRulesResponseWithoutResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PageRulesResponseWithoutResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PageRulesResponseWithoutResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class PageRuleResult():
    """
    page rule result.

    :attr str id: identifier.
    :attr List[TargetsItem] targets: targets.
    :attr List[PageRulesBodyActionsItem] actions: actions.
    :attr int priority: priority.
    :attr str status: status.
    :attr str modified_on: modified date.
    :attr str created_on: created date.
    """

    def __init__(self,
                 id: str,
                 targets: List['TargetsItem'],
                 actions: List['PageRulesBodyActionsItem'],
                 priority: int,
                 status: str,
                 modified_on: str,
                 created_on: str) -> None:
        """
        Initialize a PageRuleResult object.

        :param str id: identifier.
        :param List[TargetsItem] targets: targets.
        :param List[PageRulesBodyActionsItem] actions: actions.
        :param int priority: priority.
        :param str status: status.
        :param str modified_on: modified date.
        :param str created_on: created date.
        """
        self.id = id
        self.targets = targets
        self.actions = actions
        self.priority = priority
        self.status = status
        self.modified_on = modified_on
        self.created_on = created_on

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'PageRuleResult':
        """Initialize a PageRuleResult object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in PageRuleResult JSON')
        if 'targets' in _dict:
            args['targets'] = [TargetsItem.from_dict(x) for x in _dict.get('targets')]
        else:
            raise ValueError('Required property \'targets\' not present in PageRuleResult JSON')
        if 'actions' in _dict:
            args['actions'] = [PageRulesBodyActionsItem.from_dict(x) for x in _dict.get('actions')]
        else:
            raise ValueError('Required property \'actions\' not present in PageRuleResult JSON')
        if 'priority' in _dict:
            args['priority'] = _dict.get('priority')
        else:
            raise ValueError('Required property \'priority\' not present in PageRuleResult JSON')
        if 'status' in _dict:
            args['status'] = _dict.get('status')
        else:
            raise ValueError('Required property \'status\' not present in PageRuleResult JSON')
        if 'modified_on' in _dict:
            args['modified_on'] = _dict.get('modified_on')
        else:
            raise ValueError('Required property \'modified_on\' not present in PageRuleResult JSON')
        if 'created_on' in _dict:
            args['created_on'] = _dict.get('created_on')
        else:
            raise ValueError('Required property \'created_on\' not present in PageRuleResult JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a PageRuleResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'targets') and self.targets is not None:
            _dict['targets'] = [x.to_dict() for x in self.targets]
        if hasattr(self, 'actions') and self.actions is not None:
            _dict['actions'] = [x.to_dict() for x in self.actions]
        if hasattr(self, 'priority') and self.priority is not None:
            _dict['priority'] = self.priority
        if hasattr(self, 'status') and self.status is not None:
            _dict['status'] = self.status
        if hasattr(self, 'modified_on') and self.modified_on is not None:
            _dict['modified_on'] = self.modified_on
        if hasattr(self, 'created_on') and self.created_on is not None:
            _dict['created_on'] = self.created_on
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this PageRuleResult object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'PageRuleResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'PageRuleResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Custom Pages
"""

from datetime import datetime
from enum import Enum
from typing import Dict, List
import json

from ibm_cloud_sdk_core import BaseService, DetailedResponse
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from ibm_cloud_sdk_core.utils import datetime_to_string, string_to_datetime

from .common import get_sdk_headers

##############################################################################
# Service
##############################################################################

class CustomPagesV1(BaseService):
    """The Custom Pages V1 service."""

    DEFAULT_SERVICE_URL = 'https://api.cis.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'custom_pages'

    @classmethod
    def new_instance(cls,
                     crn: str,
                     zone_identifier: str,
                     service_name: str = DEFAULT_SERVICE_NAME,
                    ) -> 'CustomPagesV1':
        """
        Return a new client for the Custom Pages service using the specified
               parameters and external configuration.

        :param str crn: Full crn of the service instance.

        :param str zone_identifier: Zone identifier.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        authenticator = get_authenticator_from_environment(service_name)
        service = cls(
            crn,
            zone_identifier,
            authenticator
            )
        service.configure_service(service_name)
        return service

    def __init__(self,
                 crn: str,
                 zone_identifier: str,
                 authenticator: Authenticator = None,
                ) -> None:
        """
        Construct a new client for the Custom Pages service.

        :param str crn: Full crn of the service instance.

        :param str zone_identifier: Zone identifier.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if crn is None:
            raise ValueError('crn must be provided')
        if zone_identifier is None:
            raise ValueError('zone_identifier must be provided')

        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator)
        self.crn = crn
        self.zone_identifier = zone_identifier


    #########################
    # Custom Pages
    #########################


    def list_instance_custom_pages(self,
        **kwargs
    ) -> DetailedResponse:
        """
        List all custom pages for a given instance.

        List all custom pages for a given instance.

        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListCustomPagesResp` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_instance_custom_pages')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/custom_pages'.format(
            *self.encode_path_vars(self.crn))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def get_instance_custom_page(self,
        page_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get a custom page for a given instance.

        Get a specific custom page for a given instance.

        :param str page_identifier: Custom page identifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CustomPageSpecificResp` object
        """

        if page_identifier is None:
            raise ValueError('page_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_instance_custom_page')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/custom_pages/{1}'.format(
            *self.encode_path_vars(self.crn, page_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_instance_custom_page(self,
        page_identifier: str,
        *,
        url: str = None,
        state: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update a custom page for a given instance.

        Update a specific custom page for a given instance.

        :param str page_identifier: Custom page identifier.
        :param str url: (optional) A URL that is associated with the Custom Page.
        :param str state: (optional) The Custom Page state.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CustomPageSpecificResp` object
        """

        if page_identifier is None:
            raise ValueError('page_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_instance_custom_page')
        headers.update(sdk_headers)

        data = {
            'url': url,
            'state': state
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/custom_pages/{1}'.format(
            *self.encode_path_vars(self.crn, page_identifier))
        request = self.prepare_request(method='PUT',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


    def list_zone_custom_pages(self,
        **kwargs
    ) -> DetailedResponse:
        """
        List all custom pages for a given zone.

        List all custom pages for a given zone.

        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `ListCustomPagesResp` object
        """

        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='list_zone_custom_pages')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/custom_pages'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def get_zone_custom_page(self,
        page_identifier: str,
        **kwargs
    ) -> DetailedResponse:
        """
        Get a custom page for a given zone.

        Get a specific custom page for a given zone.

        :param str page_identifier: Custom page identifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CustomPageSpecificResp` object
        """

        if page_identifier is None:
            raise ValueError('page_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='get_zone_custom_page')
        headers.update(sdk_headers)

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/custom_pages/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, page_identifier))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers)

        response = self.send(request)
        return response


    def update_zone_custom_page(self,
        page_identifier: str,
        *,
        url: str = None,
        state: str = None,
        **kwargs
    ) -> DetailedResponse:
        """
        Update a custom page for a given zone.

        Update a specific custom page for a given zone.

        :param str page_identifier: Custom page identifier.
        :param str url: (optional) A URL that is associated with the Custom Page.
        :param str state: (optional) The Custom Page state.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse with `dict` result representing a `CustomPageSpecificResp` object
        """

        if page_identifier is None:
            raise ValueError('page_identifier must be provided')
        headers = {}
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V1',
                                      operation_id='update_zone_custom_page')
        headers.update(sdk_headers)

        data = {
            'url': url,
            'state': state
        }
        data = {k: v for (k, v) in data.items() if v is not None}
        data = json.dumps(data)
        headers['content-type'] = 'application/json'

        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))

        url = '/v1/{0}/zones/{1}/custom_pages/{2}'.format(
            *self.encode_path_vars(self.crn, self.zone_identifier, page_identifier))
        request = self.prepare_request(method='PUT',
                                       url=url,
                                       headers=headers,
                                       data=data)

        response = self.send(request)
        return response


class GetInstanceCustomPageEnums:
    """
    Enums for get_instance_custom_page parameters.
    """

    class PageIdentifier(str, Enum):
        """
        Custom page identifier.
        """
        BASIC_CHALLENGE = 'basic_challenge'
        WAF_CHALLENGE = 'waf_challenge'
        WAF_BLOCK = 'waf_block'
        RATELIMIT_BLOCK = 'ratelimit_block'
        COUNTRY_CHALLENGE = 'country_challenge'
        IP_BLOCK = 'ip_block'
        UNDER_ATTACK = 'under_attack'


class UpdateInstanceCustomPageEnums:
    """
    Enums for update_instance_custom_page parameters.
    """

    class PageIdentifier(str, Enum):
        """
        Custom page identifier.
        """
        BASIC_CHALLENGE = 'basic_challenge'
        WAF_CHALLENGE = 'waf_challenge'
        WAF_BLOCK = 'waf_block'
        RATELIMIT_BLOCK = 'ratelimit_block'
        COUNTRY_CHALLENGE = 'country_challenge'
        IP_BLOCK = 'ip_block'
        UNDER_ATTACK = 'under_attack'


class GetZoneCustomPageEnums:
    """
    Enums for get_zone_custom_page parameters.
    """

    class PageIdentifier(str, Enum):
        """
        Custom page identifier.
        """
        BASIC_CHALLENGE = 'basic_challenge'
        WAF_CHALLENGE = 'waf_challenge'
        WAF_BLOCK = 'waf_block'
        RATELIMIT_BLOCK = 'ratelimit_block'
        COUNTRY_CHALLENGE = 'country_challenge'
        IP_BLOCK = 'ip_block'
        UNDER_ATTACK = 'under_attack'


class UpdateZoneCustomPageEnums:
    """
    Enums for update_zone_custom_page parameters.
    """

    class PageIdentifier(str, Enum):
        """
        Custom page identifier.
        """
        BASIC_CHALLENGE = 'basic_challenge'
        WAF_CHALLENGE = 'waf_challenge'
        WAF_BLOCK = 'waf_block'
        RATELIMIT_BLOCK = 'ratelimit_block'
        COUNTRY_CHALLENGE = 'country_challenge'
        IP_BLOCK = 'ip_block'
        UNDER_ATTACK = 'under_attack'


##############################################################################
# Models
##############################################################################


class ListCustomPagesRespResultInfo():
    """
    Statistics of results.

    :attr int page: Page number.
    :attr int per_page: Number of results per page.
    :attr int total_pages: Number of total pages.
    :attr int count: Number of results.
    :attr int total_count: Total number of results.
    """

    def __init__(self,
                 page: int,
                 per_page: int,
                 total_pages: int,
                 count: int,
                 total_count: int) -> None:
        """
        Initialize a ListCustomPagesRespResultInfo object.

        :param int page: Page number.
        :param int per_page: Number of results per page.
        :param int total_pages: Number of total pages.
        :param int count: Number of results.
        :param int total_count: Total number of results.
        """
        self.page = page
        self.per_page = per_page
        self.total_pages = total_pages
        self.count = count
        self.total_count = total_count

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListCustomPagesRespResultInfo':
        """Initialize a ListCustomPagesRespResultInfo object from a json dictionary."""
        args = {}
        if 'page' in _dict:
            args['page'] = _dict.get('page')
        else:
            raise ValueError('Required property \'page\' not present in ListCustomPagesRespResultInfo JSON')
        if 'per_page' in _dict:
            args['per_page'] = _dict.get('per_page')
        else:
            raise ValueError('Required property \'per_page\' not present in ListCustomPagesRespResultInfo JSON')
        if 'total_pages' in _dict:
            args['total_pages'] = _dict.get('total_pages')
        else:
            raise ValueError('Required property \'total_pages\' not present in ListCustomPagesRespResultInfo JSON')
        if 'count' in _dict:
            args['count'] = _dict.get('count')
        else:
            raise ValueError('Required property \'count\' not present in ListCustomPagesRespResultInfo JSON')
        if 'total_count' in _dict:
            args['total_count'] = _dict.get('total_count')
        else:
            raise ValueError('Required property \'total_count\' not present in ListCustomPagesRespResultInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListCustomPagesRespResultInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'page') and self.page is not None:
            _dict['page'] = self.page
        if hasattr(self, 'per_page') and self.per_page is not None:
            _dict['per_page'] = self.per_page
        if hasattr(self, 'total_pages') and self.total_pages is not None:
            _dict['total_pages'] = self.total_pages
        if hasattr(self, 'count') and self.count is not None:
            _dict['count'] = self.count
        if hasattr(self, 'total_count') and self.total_count is not None:
            _dict['total_count'] = self.total_count
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListCustomPagesRespResultInfo object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListCustomPagesRespResultInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListCustomPagesRespResultInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class CustomPageObject():
    """
    custom page object.

    :attr str id: Custom page identifier.
    :attr str description: Description of custom page.
    :attr List[str] required_tokens: array of page tokens.
    :attr str preview_target: Preview target.
    :attr datetime created_on: Created date.
    :attr datetime modified_on: Modified date.
    :attr str url: A URL that is associated with the Custom Page.
    :attr str state: The Custom Page state.
    """

    def __init__(self,
                 id: str,
                 description: str,
                 required_tokens: List[str],
                 preview_target: str,
                 created_on: datetime,
                 modified_on: datetime,
                 url: str,
                 state: str) -> None:
        """
        Initialize a CustomPageObject object.

        :param str id: Custom page identifier.
        :param str description: Description of custom page.
        :param List[str] required_tokens: array of page tokens.
        :param str preview_target: Preview target.
        :param datetime created_on: Created date.
        :param datetime modified_on: Modified date.
        :param str url: A URL that is associated with the Custom Page.
        :param str state: The Custom Page state.
        """
        self.id = id
        self.description = description
        self.required_tokens = required_tokens
        self.preview_target = preview_target
        self.created_on = created_on
        self.modified_on = modified_on
        self.url = url
        self.state = state

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CustomPageObject':
        """Initialize a CustomPageObject object from a json dictionary."""
        args = {}
        if 'id' in _dict:
            args['id'] = _dict.get('id')
        else:
            raise ValueError('Required property \'id\' not present in CustomPageObject JSON')
        if 'description' in _dict:
            args['description'] = _dict.get('description')
        else:
            raise ValueError('Required property \'description\' not present in CustomPageObject JSON')
        if 'required_tokens' in _dict:
            args['required_tokens'] = _dict.get('required_tokens')
        else:
            raise ValueError('Required property \'required_tokens\' not present in CustomPageObject JSON')
        if 'preview_target' in _dict:
            args['preview_target'] = _dict.get('preview_target')
        else:
            raise ValueError('Required property \'preview_target\' not present in CustomPageObject JSON')
        if 'created_on' in _dict:
            args['created_on'] = string_to_datetime(_dict.get('created_on'))
        else:
            raise ValueError('Required property \'created_on\' not present in CustomPageObject JSON')
        if 'modified_on' in _dict:
            args['modified_on'] = string_to_datetime(_dict.get('modified_on'))
        else:
            raise ValueError('Required property \'modified_on\' not present in CustomPageObject JSON')
        if 'url' in _dict:
            args['url'] = _dict.get('url')
        else:
            raise ValueError('Required property \'url\' not present in CustomPageObject JSON')
        if 'state' in _dict:
            args['state'] = _dict.get('state')
        else:
            raise ValueError('Required property \'state\' not present in CustomPageObject JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CustomPageObject object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'id') and self.id is not None:
            _dict['id'] = self.id
        if hasattr(self, 'description') and self.description is not None:
            _dict['description'] = self.description
        if hasattr(self, 'required_tokens') and self.required_tokens is not None:
            _dict['required_tokens'] = self.required_tokens
        if hasattr(self, 'preview_target') and self.preview_target is not None:
            _dict['preview_target'] = self.preview_target
        if hasattr(self, 'created_on') and self.created_on is not None:
            _dict['created_on'] = datetime_to_string(self.created_on)
        if hasattr(self, 'modified_on') and self.modified_on is not None:
            _dict['modified_on'] = datetime_to_string(self.modified_on)
        if hasattr(self, 'url') and self.url is not None:
            _dict['url'] = self.url
        if hasattr(self, 'state') and self.state is not None:
            _dict['state'] = self.state
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CustomPageObject object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CustomPageObject') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CustomPageObject') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class IdEnum(str, Enum):
        """
        Custom page identifier.
        """
        BASIC_CHALLENGE = 'basic_challenge'
        WAF_CHALLENGE = 'waf_challenge'
        WAF_BLOCK = 'waf_block'
        RATELIMIT_BLOCK = 'ratelimit_block'
        COUNTRY_CHALLENGE = 'country_challenge'
        IP_BLOCK = 'ip_block'
        UNDER_ATTACK = 'under_attack'


    class RequiredTokensEnum(str, Enum):
        """
        required_tokens.
        """
        CAPTCHA_BOX = '::CAPTCHA_BOX::'
        IM_UNDER_ATTACK_BOX = '::IM_UNDER_ATTACK_BOX::'
        CLOUDFLARE_ERROR_500S_BOX = '::CLOUDFLARE_ERROR_500S_BOX::'
        CLOUDFLARE_ERROR_1000S_BOX = '::CLOUDFLARE_ERROR_1000S_BOX::'


    class StateEnum(str, Enum):
        """
        The Custom Page state.
        """
        DEFAULT = 'default'
        CUSTOMIZED = 'customized'


class CustomPageSpecificResp():
    """
    custom page specific response.

    :attr bool success: Was operation successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr CustomPageObject result: custom page object.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: 'CustomPageObject') -> None:
        """
        Initialize a CustomPageSpecificResp object.

        :param bool success: Was operation successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param CustomPageObject result: custom page object.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'CustomPageSpecificResp':
        """Initialize a CustomPageSpecificResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in CustomPageSpecificResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in CustomPageSpecificResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in CustomPageSpecificResp JSON')
        if 'result' in _dict:
            args['result'] = CustomPageObject.from_dict(_dict.get('result'))
        else:
            raise ValueError('Required property \'result\' not present in CustomPageSpecificResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a CustomPageSpecificResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = self.result.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this CustomPageSpecificResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'CustomPageSpecificResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CustomPageSpecificResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

class ListCustomPagesResp():
    """
    list of custom pages response.

    :attr bool success: Was operation successful.
    :attr List[List[str]] errors: Array of errors encountered.
    :attr List[List[str]] messages: Array of messages returned.
    :attr List[CustomPageObject] result: custom pages array.
    :attr ListCustomPagesRespResultInfo result_info: Statistics of results.
    """

    def __init__(self,
                 success: bool,
                 errors: List[List[str]],
                 messages: List[List[str]],
                 result: List['CustomPageObject'],
                 result_info: 'ListCustomPagesRespResultInfo') -> None:
        """
        Initialize a ListCustomPagesResp object.

        :param bool success: Was operation successful.
        :param List[List[str]] errors: Array of errors encountered.
        :param List[List[str]] messages: Array of messages returned.
        :param List[CustomPageObject] result: custom pages array.
        :param ListCustomPagesRespResultInfo result_info: Statistics of results.
        """
        self.success = success
        self.errors = errors
        self.messages = messages
        self.result = result
        self.result_info = result_info

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ListCustomPagesResp':
        """Initialize a ListCustomPagesResp object from a json dictionary."""
        args = {}
        if 'success' in _dict:
            args['success'] = _dict.get('success')
        else:
            raise ValueError('Required property \'success\' not present in ListCustomPagesResp JSON')
        if 'errors' in _dict:
            args['errors'] = _dict.get('errors')
        else:
            raise ValueError('Required property \'errors\' not present in ListCustomPagesResp JSON')
        if 'messages' in _dict:
            args['messages'] = _dict.get('messages')
        else:
            raise ValueError('Required property \'messages\' not present in ListCustomPagesResp JSON')
        if 'result' in _dict:
            args['result'] = [CustomPageObject.from_dict(x) for x in _dict.get('result')]
        else:
            raise ValueError('Required property \'result\' not present in ListCustomPagesResp JSON')
        if 'result_info' in _dict:
            args['result_info'] = ListCustomPagesRespResultInfo.from_dict(_dict.get('result_info'))
        else:
            raise ValueError('Required property \'result_info\' not present in ListCustomPagesResp JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ListCustomPagesResp object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'success') and self.success is not None:
            _dict['success'] = self.success
        if hasattr(self, 'errors') and self.errors is not None:
            _dict['errors'] = self.errors
        if hasattr(self, 'messages') and self.messages is not None:
            _dict['messages'] = self.messages
        if hasattr(self, 'result') and self.result is not None:
            _dict['result'] = [x.to_dict() for x in self.result]
        if hasattr(self, 'result_info') and self.result_info is not None:
            _dict['result_info'] = self.result_info.to_dict()
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ListCustomPagesResp object."""
        return json.dumps(self.to_dict(), indent=2)

    def __eq__(self, other: 'ListCustomPagesResp') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ListCustomPagesResp') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other
